@@ -32,7 +32,6 @@ from oslo_utils import uuidutils
 from ovsdbapp.backend.ovs_idl import idlutils
 from ovsdbapp import exceptions as ovs_exceptions
 
-import debtcollector
 import tenacity
 
 from neutron._i18n import _
@@ -223,8 +222,8 @@ class BaseOVS(object):
     @property
     def is_hw_offload_enabled(self):
         if self._hw_offload is None:
-            self._hw_offload = self.config.get('other_config',
-                                   {}).get('hw-offload', '').lower() == 'true'
+            self._hw_offload = self.config.get('other_config', {}).get(
+                'hw-offload', '').lower() == 'true'
         return self._hw_offload
 
 
@@ -435,8 +434,6 @@ class OVSBridge(BaseOVS):
         self.ovsdb.del_port(port_name, self.br_name).execute()
 
     def run_ofctl(self, cmd, args, process_input=None):
-        debtcollector.deprecate("Use of run_ofctl is "
-            "deprecated", removal_version='V')
         full_args = ["ovs-ofctl", cmd,
                      "-O", self._highest_protocol_needed,
                      self.br_name] + args
@@ -1170,7 +1167,7 @@ class OVSBridge(BaseOVS):
         queues = self.ovsdb.db_list(
             'Queue',
             columns=['_uuid', 'external_ids', 'other_config']).execute(
-            check_error=True)
+                check_error=True)
         if port:
             queues = [queue for queue in queues
                       if queue['external_ids'].get('port') == str(port)]
