@@ -510,7 +510,7 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         else:
             mac_ipv6_pairs.append((mac, ip_address))
             lla = str(netutils.get_ipv6_addr_by_EUI64(
-                    constants.IPv6_LLA_PREFIX, mac))
+                constants.IPv6_LLA_PREFIX, mac))
             if (mac, lla) not in mac_ipv6_pairs:
                 # only add once so we don't generate duplicate rules
                 mac_ipv6_pairs.append((mac, lla))
@@ -994,7 +994,8 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
             self._clean_deleted_remote_sg_members_conntrack_entries()
 
     def _get_sg_members(self, sg_info, sg_id, ethertype):
-        return set(sg_info.get(sg_id, {}).get(ethertype, []))
+        ip_mac_addresses = sg_info.get(sg_id, {}).get(ethertype, [])
+        return set([ip_mac[0] for ip_mac in ip_mac_addresses])
 
     def filter_defer_apply_off(self):
         if self._defer_apply:
