@@ -13,7 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import distutils
 import enum
 import re
 import shutil
@@ -22,6 +21,7 @@ import tempfile
 import netaddr
 from neutron_lib import constants as n_consts
 from neutron_lib import exceptions
+from neutron_lib.plugins.ml2 import ovs_constants as ovs_const
 from neutron_lib.utils import helpers
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -35,10 +35,9 @@ from neutron.agent.linux import external_process
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import keepalived
 from neutron.agent.linux import utils as agent_utils
+from neutron.cmd import runtime_checks
 from neutron.common import utils as common_utils
 from neutron.conf.agent.l3 import config as l3_config
-from neutron.plugins.ml2.drivers.openvswitch.agent.common \
-    import constants as ovs_const
 from neutron.privileged.agent.linux import dhcp as priv_dhcp
 
 LOG = logging.getLogger(__name__)
@@ -86,7 +85,8 @@ def _get_ovn_version(check_type):
 def ovs_vxlan_supported(from_ip='192.0.2.1', to_ip='192.0.2.2'):
     br_name = common_utils.get_rand_device_name(prefix='vxlantest-')
     port_name = common_utils.get_rand_device_name(prefix='vxlantest-')
-    with ovs_lib.OVSBridge(br_name) as br:
+    with ovs_lib.OVSBridge(br_name,
+                           datapath_type=cfg.CONF.OVS.datapath_type) as br:
         port = br.add_tunnel_port(
             port_name=port_name,
             remote_ip=from_ip,
@@ -98,7 +98,8 @@ def ovs_vxlan_supported(from_ip='192.0.2.1', to_ip='192.0.2.2'):
 def ovs_geneve_supported(from_ip='192.0.2.3', to_ip='192.0.2.4'):
     br_name = common_utils.get_rand_device_name(prefix='genevetest-')
     port_name = common_utils.get_rand_device_name(prefix='genevetest-')
-    with ovs_lib.OVSBridge(br_name) as br:
+    with ovs_lib.OVSBridge(br_name,
+                           datapath_type=cfg.CONF.OVS.datapath_type) as br:
         port = br.add_tunnel_port(
             port_name=port_name,
             remote_ip=from_ip,
@@ -120,7 +121,8 @@ def iproute2_vxlan_supported():
 def patch_supported():
     name, peer_name, patch_name = common_utils.get_related_rand_device_names(
         ['patchtest-', 'peertest0-', 'peertest1-'])
-    with ovs_lib.OVSBridge(name) as br:
+    with ovs_lib.OVSBridge(name,
+                           datapath_type=cfg.CONF.OVS.datapath_type) as br:
         port = br.add_patch_port(patch_name, peer_name)
         return port != ovs_lib.INVALID_OFPORT
 
@@ -142,7 +144,9 @@ def ofctl_arg_supported(cmd, **kwargs):
     :returns: a boolean if the supplied arguments are supported.
     """
     br_name = common_utils.get_rand_device_name(prefix='br-test-')
-    with ovs_lib.OVSBridge(br_name) as test_br:
+    with ovs_lib.OVSBridge(
+            br_name,
+            datapath_type=cfg.CONF.OVS.datapath_type) as test_br:
         full_args = ["ovs-ofctl", cmd, test_br.br_name,
                      ovs_lib._build_flow_expr_str(kwargs, cmd.split('-')[0],
                                                   False)]
@@ -244,11 +248,12 @@ def dnsmasq_version_supported():
         env = {'LC_ALL': 'C'}
         out = agent_utils.execute(cmd, addl_env=env)
         m = re.search(r"version (\d+\.\d+)", out)
-        ver = distutils.version.StrictVersion(m.group(1) if m else '0.0')
-        if ver < distutils.version.StrictVersion(MINIMUM_DNSMASQ_VERSION):
+        ver = versionutils.convert_version_to_tuple(m.group(1) if m else '0.0')
+        if ver < versionutils.convert_version_to_tuple(
+                MINIMUM_DNSMASQ_VERSION):
             return False
         if (cfg.CONF.dnsmasq_enable_addr6_list is True and
-                ver < distutils.version.StrictVersion(
+                ver < versionutils.convert_version_to_tuple(
                     DNSMASQ_VERSION_HOST_ADDR6_LIST)):
             LOG.warning('Support for multiple IPv6 addresses in host '
                         'entries was introduced in dnsmasq version '
@@ -275,7 +280,7 @@ def ovs_qos_direct_port_supported():
         ver = versionutils.convert_version_to_tuple(matched_version.group(1) if
                                                     matched_version else '0.0')
         minver = versionutils.convert_version_to_tuple(
-                 DIRECT_PORT_QOS_MIN_OVS_VERSION)
+            DIRECT_PORT_QOS_MIN_OVS_VERSION)
         if ver < minver:
             return False
     except (OSError, RuntimeError, ValueError) as e:
@@ -397,7 +402,8 @@ def keepalived_ipv6_supported():
     gw_vip = 'fdf8:f53b:82e4::10/64'
     expected_default_gw = 'fe80:f816::1'
 
-    with ovs_lib.OVSBridge(br_name) as br:
+    with ovs_lib.OVSBridge(br_name,
+                           datapath_type=cfg.CONF.OVS.datapath_type) as br:
         with KeepalivedIPv6Test(ha_port, gw_port, gw_vip,
                                 expected_default_gw) as ka:
             br.add_port(ha_port, ('type', 'internal'))
@@ -426,6 +432,14 @@ def keepalived_ipv6_supported():
     return expected_default_gw == default_gw
 
 
+def keepalived_garp_on_sighup_supported():
+    keepalived_garp_on_sighup = ('1.2.20')
+    keepalived_version = runtime_checks.get_keepalived_version()
+    if keepalived_version:
+        return keepalived_version >= keepalived_garp_on_sighup
+    return False
+
+
 def ovsdb_native_supported():
     # Running the test should ensure we are configured for OVSDB native
     try:
@@ -445,7 +459,8 @@ def ovsdb_native_supported():
 def ovs_conntrack_supported():
     br_name = common_utils.get_rand_device_name(prefix="ovs-test-")
 
-    with ovs_lib.OVSBridge(br_name) as br:
+    with ovs_lib.OVSBridge(br_name,
+                           datapath_type=cfg.CONF.OVS.datapath_type) as br:
         try:
             br.add_protocols(*["OpenFlow%d" % i for i in range(10, 15)])
         except RuntimeError as e:
@@ -591,7 +606,7 @@ def ovn_nb_db_schema_port_group_supported():
     try:
         ver = _get_ovn_version(OVNCheckType.nb_db_schema)
         minver = versionutils.convert_version_to_tuple(
-                 OVN_NB_DB_SCHEMA_PORT_GROUP)
+            OVN_NB_DB_SCHEMA_PORT_GROUP)
         if ver < minver:
             return False
     except (OSError, RuntimeError, ValueError) as e:
@@ -605,7 +620,7 @@ def ovn_nb_db_schema_stateless_nat_supported():
     try:
         ver = _get_ovn_version(OVNCheckType.nb_db_schema)
         minver = versionutils.convert_version_to_tuple(
-                 OVN_NB_DB_SCHEMA_STATELESS_NAT)
+            OVN_NB_DB_SCHEMA_STATELESS_NAT)
         if ver < minver:
             return False
     except (OSError, RuntimeError, ValueError) as e:
@@ -619,7 +634,7 @@ def ovn_sb_db_schema_virtual_port_supported():
     try:
         ver = _get_ovn_version(OVNCheckType.sb_db_schema)
         minver = versionutils.convert_version_to_tuple(
-                 OVN_SB_DB_SCHEMA_VIRTUAL_PORT)
+            OVN_SB_DB_SCHEMA_VIRTUAL_PORT)
         if ver < minver:
             return False
     except (OSError, RuntimeError, ValueError) as e:
