@@ -27,6 +27,7 @@ from sqlalchemy import or_
 
 from neutron._i18n import _
 from neutron.cmd.upgrade_checks import base
+from neutron.conf import service as conf_service
 from neutron.db.extra_dhcp_opt import models as extra_dhcp_opt_models
 from neutron.db.models import agent as agent_model
 from neutron.db.models import external_net
@@ -119,7 +120,8 @@ def get_duplicate_network_segment_count():
     query = query.group_by(
         segment.NetworkSegment.network_id,
         segment.NetworkSegment.network_type,
-        segment.NetworkSegment.physical_network
+        segment.NetworkSegment.physical_network,
+        segment.NetworkSegment.segment_index,
     )
     query = query.having(func.count() > 1)
     return query.count()
@@ -145,8 +147,7 @@ def get_fip_per_network_without_qos_policies(network_id):
     query = query.filter(and_(
         ~exists().where(qos_models.QosFIPPolicyBinding.fip_id ==
                         l3_models.FloatingIP.id),
-        l3_models.FloatingIP.floating_network_id == network_id)
-    )
+        l3_models.FloatingIP.floating_network_id == network_id))
     return query.count()
 
 
@@ -183,7 +184,7 @@ class CoreChecks(base.BaseChecks):
     @staticmethod
     def worker_count_check(checker):
 
-        if cfg.CONF.api_workers and cfg.CONF.rpc_workers:
+        if cfg.CONF.api_workers and conf_service.get_rpc_workers():
             return upgradecheck.Result(
                 upgradecheck.Code.SUCCESS, _("Number of workers already "
                                              "defined in config"))
@@ -313,10 +314,10 @@ class CoreChecks(base.BaseChecks):
     def nic_switch_agent_min_kernel_check(checker):
         # TODO(adrianc): This was introduced in U release, consider removing
         # in 1-2 cycles.
-        # Background: Issue with old kernel is appernet in CentOS 7 and older.
+        # Background: Issue with old kernel is apparent in CentOS 7 and older.
         # U release is the first release that moves from CentOS-7 to CentOS-8,
         # this was added as a "heads-up" for operators to make sure min kernel
-        # requirement is fullfiled.
+        # requirement is fulfilled.
         if not cfg.CONF.database.connection:
             return upgradecheck.Result(
                 upgradecheck.Code.WARNING,
