@@ -853,3 +853,160 @@ def get_ovn_chassis_other_config(chassis):
         return chassis.other_config
     except AttributeError:
         return chassis.external_ids
+
+
+def get_subnets_address_scopes(context, subnets, fixed_ips, ml2_plugin):
+    """Returns the IPv4 and IPv6 address scopes of several subnets.
+
+    The subnets hosted on the same network must be allocated from the same
+    subnet pool (from ``NetworkSubnetPoolAffinityError`` exception). That
+    applies per IP version (it means it is possible to have two subnet pools,
+    one for IPv4 and one for IPv6).
+
+    :param context: neutron api request context
+    :param subnets: (list of dict) subnet dictionaries
+    :param fixed_ips: (list of dict) fixed IPs of several subnets (usually
+                      belonging to a network but not mandatory)
+    :param ml2_plugin: (``Ml2Plugin``) ML2 plugin instance
+    :return: (tuple of 2 strings) IPv4 and IPv6 address scope IDs
+    """
+    address4_scope_id, address6_scope_id = '', ''
+    if not subnets:
+        return address4_scope_id, address6_scope_id
+
+    subnets_by_id = {subnet['id']: subnet for subnet in subnets}
+    for fixed_ip in fixed_ips:
+        subnet_id = fixed_ip.get('subnet_id')
+        subnet = subnets_by_id.get(subnet_id)
+        if not subnet or not subnet['subnetpool_id']:
+            continue
+
+        try:
+            subnet_pool = ml2_plugin.get_subnetpool(context,
+                                                    id=subnet['subnetpool_id'])
+            if subnet_pool['address_scope_id']:
+                if subnet_pool['ip_version'] == const.IP_VERSION_4:
+                    address4_scope_id = subnet_pool['address_scope_id']
+                else:
+                    address6_scope_id = subnet_pool['address_scope_id']
+        except n_exc.SubnetPoolNotFound:
+            # swallow the exception and just continue if the
+            # lookup failed
+            pass
+
+    return address4_scope_id, address6_scope_id
+
+
+def get_port_type_virtual_and_parents(subnets, fixed_ips, network_id, port_id,
+                                      nb_idl):
+    """Returns if a port is type virtual and its corresponding parents.
+
+    :param subnets: (list of dict) subnet dictionaries
+    :param fixed_ips: (list of dict) fixed IPs of several subnets (usually
+                      belonging to a network but not mandatory)
+    :param network_id: (string) network ID
+    :param port_id: (string) port ID
+    :param nb_idl: (``OvsdbNbOvnIdl``) OVN Northbound IDL
+    :return: (tuple, three strings) (1) the virtual type ('' if not virtual),
+             (2) the virtual IP address and (3) the virtual parents
+    """
+    port_type, virtual_ip, virtual_parents = '', None, None
+    if not subnets:
+        return port_type, virtual_ip, virtual_parents
+
+    subnets_by_id = {subnet['id'] for subnet in subnets}
+    for fixed_ip in fixed_ips:
+        if fixed_ip.get('subnet_id') not in subnets_by_id:
+            continue
+
+        # Check if the port being created is a virtual port
+        parents = get_virtual_port_parents(
+            nb_idl, fixed_ip['ip_address'], network_id, port_id)
+        if not parents:
+            continue
+
+        port_type = constants.LSP_TYPE_VIRTUAL
+        virtual_ip = fixed_ip['ip_address']
+        virtual_parents = ','.join(parents)
+        break
+
+    return port_type, virtual_ip, virtual_parents
+
+
+def determine_bind_host(sb_idl, port, port_context=None):
+    """Determine which host the port should be bound to.
+
+    Traditionally it has been Nova's responsibility to create Virtual
+    Interfaces (VIFs) as part of instance life cycle, and subsequently
+    manage plug/unplug operations on the Open vSwitch integration bridge.
+    For the traditional topology the bind host will be the same as the
+    hypervisor hosting the instance.
+
+    With the advent of SmartNIC DPUs which are connected to multiple
+    distinct CPUs we can have a topology where the instance runs on one
+    host and Open vSwitch and OVN runs on a different host, the SmartNIC
+    DPU control plane CPU.  In the SmartNIC DPU topology the bind host will
+    be different than the hypervisor host.
+
+    This helper accepts both a port Dict and optionally a PortContext
+    instance so that it can be used both before and after a port is bound.
+
+    :param sb_idl: OVN Southbound IDL
+    :type sb_idl: ``OvsdbSbOvnIdl``
+    :param port: Port Dictionary
+    :type port: Dict[str,any]
+    :param port_context: PortContext instance describing the port
+    :type port_context: api.PortContext
+    :returns: FQDN or Hostname to bind port to.
+    :rtype: str
+    :raises: n_exc.InvalidInput, RuntimeError
+    """
+    # Note that we use port_context.host below when called from bind_port
+    port = port_context.current if port_context else port
+    vnic_type = port.get(portbindings.VNIC_TYPE, portbindings.VNIC_NORMAL)
+    if vnic_type != portbindings.VNIC_REMOTE_MANAGED:
+        # The ``PortContext`` ``host`` property contains handling of
+        # special cases.
+        return port_context.host if port_context else port.get(
+            portbindings.HOST_ID, '')
+
+    bp_info = validate_and_get_data_from_binding_profile(port)
+    if constants.VIF_DETAILS_CARD_SERIAL_NUMBER in bp_info.bp_param:
+        return sb_idl.get_chassis_by_card_serial_from_cms_options(
+            bp_info.bp_param[
+                constants.VIF_DETAILS_CARD_SERIAL_NUMBER]).hostname
+    return ''
+
+
+def validate_port_binding_and_virtual_port(
+        port_context, nb_idl, ml2_plugin, port, original_port):
+    """If the port is type=virtual and it is bound, raise BadRequest"""
+    # If the port receives an update of the device ID and the binding profile
+    # host ID fields, at the same time, this is because Nova is trying to bind
+    # the port to a VM (device ID) in a host (host ID).
+    if not (port['device_id'] != original_port['device_id'] and
+            port[portbindings.HOST_ID] != original_port[portbindings.HOST_ID]):
+        return
+
+    fixed_ips = port.get('fixed_ips', [])
+    subnet_ids = set([fixed_ip['subnet_id'] for fixed_ip in fixed_ips
+                      if 'subnet_id' in fixed_ip])
+    if not subnet_ids:
+        # If the port has no fixed_ips/subnets, it cannot be virtual.
+        return
+
+    subnets = ml2_plugin.get_subnets(port_context.plugin_context,
+                                     filters={'id': list(subnet_ids)})
+    port_type, _, _ = get_port_type_virtual_and_parents(
+        subnets, fixed_ips, port['network_id'], port['id'], nb_idl)
+    if port_type == constants.LSP_TYPE_VIRTUAL:
+        raise n_exc.BadRequest(
+            resource='port',
+            msg='A virtual logical switch port cannot be bound to a host')
+
+
+def get_requested_chassis(requested_chassis):
+    """Returns a list with the items in the LSP.options:requested-chassis"""
+    if isinstance(requested_chassis, str):
+        return requested_chassis.split(',')
+    return []
