@@ -24,13 +24,13 @@ ASSOC_COLLECTION_PATH = '/flavors/{flavor_id}/service_profiles'
 ASSOC_RESOURCE_PATH = '/flavors/{flavor_id}/service_profiles/{profile_id}'
 
 DEPRECATION_REASON = (
-    "The flavor API now supports system scope and default roles.")
+    "The flavor API now supports project scope and default roles.")
 
 
 rules = [
     policy.DocumentedRuleDefault(
         name='create_flavor',
-        check_str=base.SYSTEM_ADMIN,
+        check_str=base.ADMIN,
         description='Create a flavor',
         operations=[
             {
@@ -38,7 +38,7 @@ rules = [
                 'path': FLAVOR_COLLECTION_PATH,
             },
         ],
-        scope_types=['system'],
+        scope_types=['project'],
         deprecated_rule=policy.DeprecatedRule(
             name='create_flavor',
             check_str=base.RULE_ADMIN_ONLY,
@@ -47,7 +47,11 @@ rules = [
     ),
     policy.DocumentedRuleDefault(
         name='get_flavor',
-        check_str=base.SYSTEM_OR_PROJECT_READER,
+        # NOTE: it can't be ADMIN_OR_PROJECT_READER constant from the base
+        # module because that is using "project_id" in the check string and the
+        # service_provider resource don't belongs to any project thus such
+        # check string would fail enforcement.
+        check_str='role:reader',
         description='Get a flavor',
         operations=[
             {
@@ -59,7 +63,7 @@ rules = [
                 'path': FLAVOR_RESOURCE_PATH,
             },
         ],
-        scope_types=['system', 'project'],
+        scope_types=['project'],
         deprecated_rule=policy.DeprecatedRule(
             name='get_flavor',
             check_str=base.RULE_ANY,
@@ -68,7 +72,7 @@ rules = [
     ),
     policy.DocumentedRuleDefault(
         name='update_flavor',
-        check_str=base.SYSTEM_ADMIN,
+        check_str=base.ADMIN,
         description='Update a flavor',
         operations=[
             {
@@ -76,7 +80,7 @@ rules = [
                 'path': FLAVOR_RESOURCE_PATH,
             },
         ],
-        scope_types=['system'],
+        scope_types=['project'],
         deprecated_rule=policy.DeprecatedRule(
             name='update_flavor',
             check_str=base.RULE_ADMIN_ONLY,
@@ -85,7 +89,7 @@ rules = [
     ),
     policy.DocumentedRuleDefault(
         name='delete_flavor',
-        check_str=base.SYSTEM_ADMIN,
+        check_str=base.ADMIN,
         description='Delete a flavor',
         operations=[
             {
@@ -93,7 +97,7 @@ rules = [
                 'path': FLAVOR_RESOURCE_PATH,
             },
         ],
-        scope_types=['system'],
+        scope_types=['project'],
         deprecated_rule=policy.DeprecatedRule(
             name='delete_flavor',
             check_str=base.RULE_ADMIN_ONLY,
@@ -103,7 +107,7 @@ rules = [
 
     policy.DocumentedRuleDefault(
         name='create_service_profile',
-        check_str=base.SYSTEM_ADMIN,
+        check_str=base.ADMIN,
         description='Create a service profile',
         operations=[
             {
@@ -111,7 +115,7 @@ rules = [
                 'path': PROFILE_COLLECTION_PATH,
             },
         ],
-        scope_types=['system'],
+        scope_types=['project'],
         deprecated_rule=policy.DeprecatedRule(
             name='create_service_profile',
             check_str=base.RULE_ADMIN_ONLY,
@@ -120,7 +124,7 @@ rules = [
     ),
     policy.DocumentedRuleDefault(
         name='get_service_profile',
-        check_str=base.SYSTEM_READER,
+        check_str=base.ADMIN,
         description='Get a service profile',
         operations=[
             {
@@ -132,7 +136,7 @@ rules = [
                 'path': PROFILE_RESOURCE_PATH,
             },
         ],
-        scope_types=['system'],
+        scope_types=['project'],
         deprecated_rule=policy.DeprecatedRule(
             name='get_service_profile',
             check_str=base.RULE_ADMIN_ONLY,
@@ -141,7 +145,7 @@ rules = [
     ),
     policy.DocumentedRuleDefault(
         name='update_service_profile',
-        check_str=base.SYSTEM_ADMIN,
+        check_str=base.ADMIN,
         description='Update a service profile',
         operations=[
             {
@@ -149,7 +153,7 @@ rules = [
                 'path': PROFILE_RESOURCE_PATH,
             },
         ],
-        scope_types=['system'],
+        scope_types=['project'],
         deprecated_rule=policy.DeprecatedRule(
             name='update_service_profile',
             check_str=base.RULE_ADMIN_ONLY,
@@ -158,7 +162,7 @@ rules = [
     ),
     policy.DocumentedRuleDefault(
         name='delete_service_profile',
-        check_str=base.SYSTEM_ADMIN,
+        check_str=base.ADMIN,
         description='Delete a service profile',
         operations=[
             {
@@ -166,7 +170,7 @@ rules = [
                 'path': PROFILE_RESOURCE_PATH,
             },
         ],
-        scope_types=['system'],
+        scope_types=['project'],
         deprecated_rule=policy.DeprecatedRule(
             name='delete_service_profile',
             check_str=base.RULE_ADMIN_ONLY,
@@ -176,13 +180,13 @@ rules = [
 
     policy.RuleDefault(
         name='get_flavor_service_profile',
-        check_str=base.SYSTEM_OR_PROJECT_READER,
+        check_str=base.ADMIN_OR_PROJECT_READER,
         description=(
             'Get a flavor associated with a given service profiles. '
             'There is no corresponding GET operations in API currently. '
             'This rule is currently referred only in the DELETE '
             'of flavor_service_profile.'),
-        scope_types=['system', 'project'],
+        scope_types=['project'],
         deprecated_rule=policy.DeprecatedRule(
             name='get_flavor_service_profile',
             check_str=base.RULE_ANY,
@@ -191,7 +195,7 @@ rules = [
     ),
     policy.DocumentedRuleDefault(
         name='create_flavor_service_profile',
-        check_str=base.SYSTEM_ADMIN,
+        check_str=base.ADMIN,
         description='Associate a flavor with a service profile',
         operations=[
             {
@@ -199,7 +203,7 @@ rules = [
                 'path': ASSOC_COLLECTION_PATH,
             },
         ],
-        scope_types=['system'],
+        scope_types=['project'],
         deprecated_rule=policy.DeprecatedRule(
             name='create_flavor_service_profile',
             check_str=base.RULE_ADMIN_ONLY,
@@ -208,7 +212,7 @@ rules = [
     ),
     policy.DocumentedRuleDefault(
         name='delete_flavor_service_profile',
-        check_str=base.SYSTEM_ADMIN,
+        check_str=base.ADMIN,
         description='Disassociate a flavor with a service profile',
         operations=[
             {
@@ -216,7 +220,7 @@ rules = [
                 'path': ASSOC_RESOURCE_PATH,
             },
         ],
-        scope_types=['system'],
+        scope_types=['project'],
         deprecated_rule=policy.DeprecatedRule(
             name='delete_flavor_service_profile',
             check_str=base.RULE_ADMIN_ONLY,
