@@ -36,7 +36,7 @@ rules = [
 
     policy.DocumentedRuleDefault(
         name='create_rbac_policy',
-        check_str=base.PROJECT_MEMBER,
+        check_str=base.ADMIN_OR_PROJECT_MEMBER,
         scope_types=['project'],
         description='Create an RBAC policy',
         operations=[
@@ -56,7 +56,7 @@ rules = [
     policy.DocumentedRuleDefault(
         name='create_rbac_policy:target_tenant',
         check_str=base.policy_or(
-            base.PROJECT_ADMIN,
+            base.ADMIN,
             '(not field:rbac_policy:target_tenant=* and '
             'not field:rbac_policy:target_project=*)'),
         description='Specify ``target_tenant`` when creating an RBAC policy',
@@ -75,7 +75,7 @@ rules = [
     ),
     policy.DocumentedRuleDefault(
         name='update_rbac_policy',
-        check_str=base.PROJECT_MEMBER,
+        check_str=base.ADMIN_OR_PROJECT_MEMBER,
         scope_types=['project'],
         description='Update an RBAC policy',
         operations=[
@@ -95,7 +95,7 @@ rules = [
     policy.DocumentedRuleDefault(
         name='update_rbac_policy:target_tenant',
         check_str=base.policy_or(
-            base.PROJECT_ADMIN,
+            base.ADMIN,
             '(not field:rbac_policy:target_tenant=* and '
             'not field:rbac_policy:target_project=*)'),
         description='Update ``target_tenant`` attribute of an RBAC policy',
@@ -116,7 +116,7 @@ rules = [
     ),
     policy.DocumentedRuleDefault(
         name='get_rbac_policy',
-        check_str=base.PROJECT_READER,
+        check_str=base.ADMIN_OR_PROJECT_READER,
         scope_types=['project'],
         description='Get an RBAC policy',
         operations=[
@@ -137,7 +137,7 @@ rules = [
     ),
     policy.DocumentedRuleDefault(
         name='delete_rbac_policy',
-        check_str=base.PROJECT_MEMBER,
+        check_str=base.ADMIN_OR_PROJECT_MEMBER,
         scope_types=['project'],
         description='Delete an RBAC policy',
         operations=[
