@@ -36,6 +36,8 @@ VICTORIA = 'victoria'
 WALLABY = 'wallaby'
 XENA = 'xena'
 YOGA = 'yoga'
+ZED = 'zed'
+RELEASE_2023_1 = '2023.1'
 
 NEUTRON_MILESTONES = [
     # earlier milestones were not tagged
@@ -52,6 +54,8 @@ NEUTRON_MILESTONES = [
     VICTORIA,
     WALLABY,
     XENA,
+    YOGA,
+    RELEASE_2023_1,
     # Do not add the milestone until the end of the release
 ]
 
