@@ -27,6 +27,7 @@ from oslo_utils import fileutils
 from oslo_utils import importutils
 
 from neutron._i18n import _
+from neutron.common import config as common_config
 from neutron.conf.db import migration_cli
 from neutron.db import migration
 from neutron.db.migration.connection import DBConnection
@@ -37,7 +38,7 @@ HEADS_FILENAME = 'HEADS'
 CONTRACT_HEAD_FILENAME = 'CONTRACT_HEAD'
 EXPAND_HEAD_FILENAME = 'EXPAND_HEAD'
 
-CURRENT_RELEASE = migration.YOGA
+CURRENT_RELEASE = migration.RELEASE_2023_1
 RELEASES = (
     migration.LIBERTY,
     migration.MITAKA,
@@ -53,6 +54,8 @@ RELEASES = (
     migration.WALLABY,
     migration.XENA,
     migration.YOGA,
+    migration.ZED,
+    migration.RELEASE_2023_1,
 )
 
 EXPAND_BRANCH = 'expand'
@@ -650,6 +653,7 @@ def get_engine_config():
 
 
 def main():
+    common_config.register_common_config_options()
     # Interpret the config file for Python logging.
     # This line sets up loggers basically.
     logging_config.fileConfig(neutron_alembic_ini)
