@@ -53,7 +53,8 @@ class QuotaEnforcementHook(hooks.PecanHook):
                 LOG.debug("Made reservation on behalf of %(tenant_id)s "
                           "for: %(delta)s",
                           {'tenant_id': tenant_id, 'delta': {resource: delta}})
-                reservations.append(reservation)
+                if reservation:
+                    reservations.append(reservation)
             except exceptions.QuotaResourceUnknown as e:
                 # Quotas cannot be enforced on this resource
                 LOG.debug(e)
@@ -61,6 +62,7 @@ class QuotaEnforcementHook(hooks.PecanHook):
         # retrieved in the 'after' hook
         state.request.context['reservations'] = reservations
 
+    @db_api.retry_db_errors
     def after(self, state):
         neutron_context = state.request.context.get('neutron_context')
         if not neutron_context:
@@ -75,9 +77,9 @@ class QuotaEnforcementHook(hooks.PecanHook):
         reservations = state.request.context.get('reservations') or []
         if not reservations and state.request.method != 'DELETE':
             return
-        with db_api.CONTEXT_WRITER.using(neutron_context):
-            # Commit the reservation(s)
-            for reservation in reservations:
-                quota.QUOTAS.commit_reservation(
-                    neutron_context, reservation.reservation_id)
-            resource_registry.set_resources_dirty(neutron_context)
+
+        # Commit the reservation(s)
+        for reservation in reservations:
+            quota.QUOTAS.commit_reservation(
+                neutron_context, reservation.reservation_id)
+        resource_registry.set_resources_dirty(neutron_context)
