@@ -15,6 +15,7 @@
 from neutron.objects.qos import binding as qos_binding
 from neutron.objects.qos import policy as qos_policy
 from neutron.objects.qos import rule as qos_rule
+from neutron_lib.api.definitions import l3 as l3_api
 from neutron_lib import constants
 from neutron_lib import context as n_context
 from neutron_lib.plugins import constants as plugins_const
@@ -24,11 +25,13 @@ from oslo_log import log as logging
 
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import utils
+from neutron.common import utils as n_utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 
 
 LOG = logging.getLogger(__name__)
 OVN_QOS_DEFAULT_RULE_PRIORITY = 2002
+_MIN_RATE = ovn_const.LSP_OPTIONS_QOS_MIN_RATE
 
 
 class OVNClientQosExtension(object):
@@ -90,9 +93,13 @@ class OVNClientQosExtension(object):
                 r = {rule.rule_type: {'dscp_mark': rule.dscp_mark}}
                 qos_rules[constants.EGRESS_DIRECTION].update(r)
             elif isinstance(rule, qos_rule.QosMinimumBandwidthRule):
-                # Rule supported for Placement scheduling but not enforced in
-                # the driver.
-                pass
+                if rule.direction == constants.INGRESS_DIRECTION:
+                    LOG.warning('ML2/OVN QoS driver does not support minimum '
+                                'bandwidth rules enforcement with ingress '
+                                'direction')
+                else:
+                    r = {rule.rule_type: {'min_kbps': rule.min_kbps}}
+                    qos_rules[constants.EGRESS_DIRECTION].update(r)
             else:
                 LOG.warning('Rule type %(rule_type)s from QoS policy '
                             '%(policy_id)s is not supported in OVN',
@@ -118,7 +125,7 @@ class OVNClientQosExtension(object):
 
     def _ovn_qos_rule(self, rules_direction, rules, port_id, network_id,
                       fip_id=None, ip_address=None, resident_port=None,
-                      delete=False):
+                      router_id=None, delete=False):
         """Generate an OVN QoS register based on several Neutron QoS rules
 
         A OVN QoS register can contain "bandwidth" and "action" parameters.
@@ -131,7 +138,8 @@ class OVNClientQosExtension(object):
 
         :param rules_direction: (string) rules direction (egress, ingress).
         :param rules: (dict) {bw_limit: {max_kbps, max_burst_kbps},
-                              dscp: {dscp_mark}}
+                              dscp: {dscp_mark},
+                              minimum_bandwidth: {min_kbps}}
         :param port_id: (string) port ID; for L3 floating IP bandwidth
                         limit this is the router gateway port ID.
         :param network_id: (string) network ID.
@@ -142,6 +150,8 @@ class OVNClientQosExtension(object):
         :param resident_port: (string) for L3 floating IP bandwidth, this is
                               a logical switch port located in the chassis
                               where the floating IP traffic is NATed.
+        :param router_id: (string) router ID, for L3 router gateway port
+                          bandwidth limit.
         :param delete: (bool) defines if this rule if going to be a partial
                        one (without any bandwidth or DSCP information) to be
                        used only as deletion rule.
@@ -168,8 +178,13 @@ class OVNClientQosExtension(object):
 
         # All OVN QoS rules have an external ID reference to the port or the
         # FIP that are attached to.
+        # 1) L3 floating IP ports.
         if fip_id:
             key, value = ovn_const.OVN_FIP_EXT_ID_KEY, fip_id
+        # 2) L3 router gateway port.
+        elif router_id:
+            key, value = ovn_const.OVN_ROUTER_ID_EXT_ID_KEY, router_id
+        # 3) Fixed IP ports (aka VM ports)
         else:
             key, value = ovn_const.OVN_PORT_EXT_ID_KEY, port_id
         ovn_qos_rule['external_ids'] = {key: value}
@@ -181,9 +196,38 @@ class OVNClientQosExtension(object):
                     ovn_qos_rule['burst'] = rule['max_burst_kbps']
             elif rule_type == qos_consts.RULE_TYPE_DSCP_MARKING:
                 ovn_qos_rule.update({'dscp': rule['dscp_mark']})
+            elif rule_type == qos_consts.RULE_TYPE_MINIMUM_BANDWIDTH:
+                # NOTE(ralonsoh): minimum bandwidth rules are only supported
+                # for fixed IP ports (although this check is redundant, that
+                # ensures only fixed IP ports have this rule type in the
+                # returned dictionary).
+                if key == ovn_const.OVN_PORT_EXT_ID_KEY:
+                    ovn_qos_rule[_MIN_RATE] = str(rule['min_kbps'])
 
         return ovn_qos_rule
 
+    def _update_lsp_qos_options(self, txn, lsp, port_id, min_qos_value):
+        """Update the LSP QoS options
+
+        :param txn: the ovsdbapp transaction object.
+        :param lsp: (AddLSwitchPortCommand) logical switch port command, passed
+                    when the port is being created. Because this method is
+                    called inside the OVN DB transaction, the LSP has not been
+                    created yet nor update in the IDL local cache.
+        :param port_id: (str) Neutron port ID that matches the LSP.name.
+                        If the port ID is None, the OVN QoS rule does not
+                        apply to a LSP but to a router gateway port or a
+                        floating IP.
+        :param min_qos_value: (str) minimum bandwidth rule value in kbps; it is
+                              a string because LSP.options is a {str:str} dict.
+        """
+        lsp = lsp or self.nb_idl.lsp_get(port_id).execute()
+        if not lsp:
+            return
+
+        options = {_MIN_RATE: min_qos_value}
+        txn.add(self.nb_idl.update_lswitch_qos_options(lsp, **options))
+
     @staticmethod
     def port_effective_qos_policy_id(port):
         """Return port effective QoS policy
@@ -191,9 +235,8 @@ class OVNClientQosExtension(object):
         If the port does not have any QoS policy reference or is a network
         device, then return None.
         """
-        policy_exists = bool(port.get('qos_policy_id') or
-                             port.get('qos_network_policy_id'))
-        if not policy_exists or utils.is_network_device_port(port):
+        policy_id = n_utils.effective_qos_policy_id(port)
+        if not policy_id or utils.is_network_device_port(port):
             return None, None
 
         if port.get('qos_policy_id'):
@@ -201,16 +244,21 @@ class OVNClientQosExtension(object):
         else:
             return port['qos_network_policy_id'], 'network'
 
-    def _delete_port_qos_rules(self, txn, port_id, network_id):
+    def _delete_port_qos_rules(self, txn, port_id, network_id, lsp=None,
+                               port_deleted=False):
         # Generate generic deletion rules for both directions. In case of
         # creating deletion rules, the rule content is irrelevant.
         for ovn_rule in [self._ovn_qos_rule(direction, {}, port_id,
                                             network_id, delete=True)
                          for direction in constants.VALID_DIRECTIONS]:
+            min_qos_value = ovn_rule.pop(_MIN_RATE, None)
             txn.add(self.nb_idl.qos_del(**ovn_rule))
+            if not port_deleted:
+                self._update_lsp_qos_options(txn, lsp, port_id,
+                                             min_qos_value)
 
     def _add_port_qos_rules(self, txn, port_id, network_id, qos_policy_id,
-                            qos_rules):
+                            qos_rules, lsp=None):
         # NOTE(ralonsoh): we don't use the transaction context because the
         # QoS policy could belong to another user (network QoS policy).
         admin_context = n_context.get_admin_context()
@@ -223,6 +271,7 @@ class OVNClientQosExtension(object):
             # generate a "ovn_rule" to be used as input in a "qos_del" method.
             ovn_rule = self._ovn_qos_rule(direction, rules, port_id,
                                           network_id, delete=not rules)
+            min_qos_value = ovn_rule.pop(_MIN_RATE, None)
             if rules:
                 # NOTE(ralonsoh): with "may_exist=True", the "qos_add" will
                 # create the QoS OVN rule or update the existing one.
@@ -230,23 +279,25 @@ class OVNClientQosExtension(object):
             else:
                 # Delete, if exists, the QoS rule in this direction.
                 txn.add(self.nb_idl.qos_del(**ovn_rule, if_exists=True))
+            self._update_lsp_qos_options(txn, lsp, port_id, min_qos_value)
 
     def _update_port_qos_rules(self, txn, port_id, network_id, qos_policy_id,
-                               qos_rules):
+                               qos_rules, lsp=None, port_deleted=False):
         if not qos_policy_id:
-            self._delete_port_qos_rules(txn, port_id, network_id)
+            self._delete_port_qos_rules(txn, port_id, network_id, lsp=lsp,
+                                        port_deleted=port_deleted)
         else:
             self._add_port_qos_rules(txn, port_id, network_id, qos_policy_id,
-                                     qos_rules)
+                                     qos_rules, lsp=lsp)
 
-    def create_port(self, txn, port):
-        self.update_port(txn, port, None, reset=True)
+    def create_port(self, txn, port, lsp):
+        self.update_port(txn, port, None, reset=True, lsp=lsp)
 
     def delete_port(self, txn, port):
         self.update_port(txn, port, None, delete=True)
 
     def update_port(self, txn, port, original_port, reset=False, delete=False,
-                    qos_rules=None):
+                    qos_rules=None, lsp=None):
         if utils.is_port_external(port):
             # External ports (SR-IOV) QoS is handled by the SR-IOV agent QoS
             # extension.
@@ -268,23 +319,25 @@ class OVNClientQosExtension(object):
                 return  # No QoS policy change
 
         self._update_port_qos_rules(txn, port['id'], port['network_id'],
-                                    qos_policy_id, qos_rules)
+                                    qos_policy_id, qos_rules, lsp=lsp,
+                                    port_deleted=delete)
 
     def update_network(self, txn, network, original_network, reset=False,
                        qos_rules=None):
         updated_port_ids = set([])
         updated_fip_ids = set([])
+        updated_router_ids = set([])
         if not reset and not original_network:
             # If there is no information about the previous QoS policy, do not
             # make any change.
-            return updated_port_ids, updated_fip_ids
+            return updated_port_ids, updated_fip_ids, updated_router_ids
 
         qos_policy_id = network.get('qos_policy_id')
         if not reset:
             original_qos_policy_id = original_network.get('qos_policy_id')
             if qos_policy_id == original_qos_policy_id:
                 # No QoS policy change
-                return updated_port_ids, updated_fip_ids
+                return updated_port_ids, updated_fip_ids, updated_router_ids
 
         # NOTE(ralonsoh): we don't use the transaction context because some
         # ports can belong to other projects.
@@ -307,7 +360,14 @@ class OVNClientQosExtension(object):
             self.update_floatingip(txn, floatingip)
             updated_fip_ids.add(floatingip['id'])
 
-        return updated_port_ids, updated_fip_ids
+        for router in (qos_binding.QosPolicyRouterGatewayIPBinding.
+                       get_routers_by_network_id(admin_context,
+                                                 network['id'])):
+            router_dict = self._plugin_l3._make_router_dict(router)
+            self.update_router(txn, router_dict)
+            updated_router_ids.add(router.id)
+
+        return updated_port_ids, updated_fip_ids, updated_router_ids
 
     def _delete_fip_qos_rules(self, txn, fip_id, network_id):
         if network_id:
@@ -365,12 +425,58 @@ class OVNClientQosExtension(object):
     def disassociate_floatingip(self, txn, floatingip):
         self.delete_floatingip(txn, floatingip)
 
+    def _delete_gateway_ip_qos_rules(self, txn, router_id, network_id):
+        if network_id:
+            lswitch_name = utils.ovn_name(network_id)
+            txn.add(self.nb_idl.qos_del_ext_ids(
+                lswitch_name,
+                {ovn_const.OVN_ROUTER_ID_EXT_ID_KEY: router_id}))
+
+    def create_router(self, txn, router):
+        self.update_router(txn, router)
+
+    def update_router(self, txn, router):
+        gw_info = router.get(l3_api.EXTERNAL_GW_INFO) or {}
+        qos_policy_id = n_utils.effective_qos_policy_id(router)
+        router_id = router.get('id')
+        gw_port_id = router.get('gw_port_id')
+        gw_network_id = gw_info.get('network_id')
+        if not (router_id and gw_port_id and gw_network_id):
+            # NOTE(ralonsoh): when the gateway network is detached, the gateway
+            # port is deleted. Any QoS policy related to this port_id is
+            # deleted in "self.update_port()".
+            LOG.debug('Router %s does not have ID or gateway assigned',
+                      router_id)
+            return
+
+        admin_context = n_context.get_admin_context()
+        qos_rules = self._qos_rules(admin_context, qos_policy_id)
+        for direction, rules in qos_rules.items():
+            # "delete=not rule": that means, when we don't have rules, we
+            # generate a "ovn_rule" to be used as input in a "qos_del" method.
+            ovn_rule = self._ovn_qos_rule(
+                direction, rules, gw_port_id, gw_network_id,
+                router_id=router_id, delete=not rules)
+            if rules:
+                # NOTE(ralonsoh): with "may_exist=True", the "qos_add" will
+                # create the QoS OVN rule or update the existing one.
+                txn.add(self.nb_idl.qos_add(**ovn_rule, may_exist=True))
+            else:
+                # Delete, if exists, the QoS rule in this direction.
+                txn.add(self.nb_idl.qos_del(**ovn_rule, if_exists=True))
+
+    def delete_router(self, txn, router):
+        self._delete_gateway_ip_qos_rules(txn, router['id'],
+                                          router['gw_network_id'])
+
     def update_policy(self, context, policy):
         updated_port_ids = set([])
         updated_fip_ids = set([])
+        updated_router_ids = set([])
         bound_networks = policy.get_bound_networks()
         bound_ports = policy.get_bound_ports()
         bound_fips = policy.get_bound_floatingips()
+        bound_routers = policy.get_bound_routers()
         qos_rules = self._qos_rules(context, policy.id)
         # TODO(ralonsoh): we need to benchmark this transaction in systems with
         # a huge amount of ports. This can take a while and could block other
@@ -378,10 +484,11 @@ class OVNClientQosExtension(object):
         with self.nb_idl.transaction(check_error=True) as txn:
             for network_id in bound_networks:
                 network = {'qos_policy_id': policy.id, 'id': network_id}
-                port_ids, fip_ids = self.update_network(
+                port_ids, fip_ids, router_ids = self.update_network(
                     txn, network, {}, reset=True, qos_rules=qos_rules)
                 updated_port_ids.update(port_ids)
                 updated_fip_ids.update(fip_ids)
+                updated_router_ids.update(router_ids)
 
             # Update each port bound to this policy, not handled previously in
             # the network update loop
@@ -399,3 +506,10 @@ class OVNClientQosExtension(object):
                 for fip in self._plugin_l3.get_floatingips(
                         context, filters={'id': fip_ids}):
                     self.update_floatingip(txn, fip)
+
+            router_ids = [r for r in bound_routers if
+                          r not in updated_router_ids]
+            if router_ids:
+                for router in self._plugin_l3.get_routers(
+                        context, filters={'id': router_ids}):
+                    self.update_router(txn, router)
