@@ -14,19 +14,20 @@ import errno
 import os
 import socket
 
+import netaddr
 from neutron_lib import constants
 from oslo_log import log as logging
-import pyroute2
-from pyroute2 import netlink  # pylint: disable=no-name-in-module
-from pyroute2.netlink import exceptions \
-    as netlink_exceptions  # pylint: disable=no-name-in-module
-from pyroute2.netlink import rtnl  # pylint: disable=no-name-in-module
-from pyroute2.netlink.rtnl import ifinfmsg  # pylint: disable=no-name-in-module
-from pyroute2.netlink.rtnl import ndmsg  # pylint: disable=no-name-in-module
-from pyroute2 import netns  # pylint: disable=no-name-in-module
+from pyroute2 import iproute
+from pyroute2.netlink import exceptions as netlink_exceptions
+from pyroute2.netlink import rtnl
+from pyroute2.netlink.rtnl import ifinfmsg
+from pyroute2.netlink.rtnl import ndmsg
+from pyroute2 import netns
+from pyroute2.nslink import nslink
 import tenacity
 
 from neutron._i18n import _
+from neutron.common import utils as common_utils
 from neutron import privileged
 from neutron.privileged.agent import linux as priv_linux
 
@@ -72,7 +73,7 @@ class NetworkInterfaceNotFound(RuntimeError):
         # and will call it always with passing only message from originally
         # raised exception.
         message = message or self.message % {
-                'device': device, 'namespace': namespace}
+            'device': device, 'namespace': namespace}
         super(NetworkInterfaceNotFound, self).__init__(message)
 
 
@@ -87,7 +88,7 @@ class InterfaceOperationNotSupported(RuntimeError):
         # and will call it always with passing only message from originally
         # raised exception.
         message = message or self.message % {
-                'device': device, 'namespace': namespace}
+            'device': device, 'namespace': namespace}
         super(InterfaceOperationNotSupported, self).__init__(message)
 
 
@@ -145,15 +146,15 @@ def get_iproute(namespace):
     # `NetNS` -- RTNL API to another network namespace
     if namespace:
         # do not try and create the namespace
-        return pyroute2.NetNS(namespace, flags=0, libc=priv_linux.get_cdll())
+        return nslink.NetNS(namespace, flags=0, libc=priv_linux.get_cdll())
     else:
-        return pyroute2.IPRoute()
+        return iproute.IPRoute()
 
 
 @privileged.default.entrypoint
 def open_namespace(namespace):
     """Open namespace to test if the namespace is ready to be manipulated"""
-    with pyroute2.NetNS(namespace, flags=0):
+    with nslink.NetNS(namespace, flags=0):
         pass
 
 
@@ -211,7 +212,7 @@ def _run_iproute_link(command, device, namespace=None, **kwargs):
         with get_iproute(namespace) as ip:
             idx = get_link_id(device, namespace)
             return ip.link(command, index=idx, **kwargs)
-    except netlink.NetlinkError as e:
+    except netlink_exceptions.NetlinkError as e:
         _translate_ip_device_exception(e, device, namespace)
         raise
     except OSError as e:
@@ -272,6 +273,32 @@ def add_ip_address(ip_version, ip, prefixlen, device, namespace, scope,
         raise
 
 
+@privileged.default.entrypoint
+def add_ip_addresses(cidrs, device, namespace, scope,
+                     add_broadcast=True):
+    for cidr in cidrs:
+        net = netaddr.IPNetwork(cidr)
+        ip = str(net.ip)
+        prefixlen = net.prefixlen
+        family = _IP_VERSION_FAMILY_MAP[net.version]
+        broadcast = None
+        if add_broadcast:
+            broadcast = common_utils.cidr_broadcast_address_alternative(cidr)
+        try:
+            _run_iproute_addr('add',
+                              device,
+                              namespace,
+                              address=ip,
+                              mask=prefixlen,
+                              family=family,
+                              broadcast=broadcast,
+                              scope=get_scope_name(scope))
+        except netlink_exceptions.NetlinkError as e:
+            if e.code == errno.EEXIST:
+                raise IpAddressAlreadyExists(ip=ip, device=device)
+            raise
+
+
 @privileged.default.entrypoint
 def delete_ip_address(ip_version, ip, prefixlen, device, namespace):
     family = _IP_VERSION_FAMILY_MAP[ip_version]
@@ -292,6 +319,31 @@ def delete_ip_address(ip_version, ip, prefixlen, device, namespace):
         raise
 
 
+@privileged.default.entrypoint
+def delete_ip_addresses(cidrs, device, namespace):
+    for cidr in cidrs:
+        net = netaddr.IPNetwork(cidr)
+        ip = str(net.ip)
+        prefixlen = net.prefixlen
+        family = _IP_VERSION_FAMILY_MAP[net.version]
+        try:
+            _run_iproute_addr("delete",
+                              device,
+                              namespace,
+                              address=ip,
+                              mask=prefixlen,
+                              family=family)
+        except netlink_exceptions.NetlinkError as e:
+            # when trying to delete a non-existent IP address, pyroute2 raises
+            # NetlinkError with code EADDRNOTAVAIL (99, 'Cannot assign
+            # requested address')
+            # this shouldn't raise an error
+            if e.code == errno.EADDRNOTAVAIL:
+                pass
+            else:
+                raise
+
+
 @privileged.default.entrypoint
 def flush_ip_addresses(ip_version, device, namespace):
     family = _IP_VERSION_FAMILY_MAP[ip_version]
@@ -314,7 +366,7 @@ def create_interface(ifname, namespace, kind, **kwargs):
             if physical_interface:
                 link_key = "vxlan_link" if kind == "vxlan" else "link"
                 kwargs[link_key] = get_link_id(physical_interface, namespace)
-            return ip.link("add", ifname=ifname, kind=kind, **kwargs)
+            ip.link("add", ifname=ifname, kind=kind, **kwargs)
     except netlink_exceptions.NetlinkError as e:
         if e.code == errno.EEXIST:
             raise InterfaceAlreadyExists(device=ifname)
@@ -345,36 +397,35 @@ def interface_exists(ifname, namespace):
 def set_link_flags(device, namespace, flags):
     link = _run_iproute_link("get", device, namespace)[0]
     new_flags = flags | link['flags']
-    return _run_iproute_link("set", device, namespace, flags=new_flags)
+    _run_iproute_link("set", device, namespace, flags=new_flags)
 
 
 @privileged.link_cmd.entrypoint
 def set_link_attribute(device, namespace, **attributes):
-    return _run_iproute_link("set", device, namespace, **attributes)
+    _run_iproute_link("set", device, namespace, **attributes)
 
 
 @privileged.link_cmd.entrypoint
 def set_link_vf_feature(device, namespace, vf_config):
-    return _run_iproute_link("set", device, namespace=namespace, vf=vf_config)
+    _run_iproute_link("set", device, namespace=namespace, vf=vf_config)
 
 
 @privileged.link_cmd.entrypoint
 def set_link_bridge_forward_delay(device, forward_delay, namespace=None):
-    return _run_iproute_link('set', device, namespace=namespace,
-                             kind='bridge', br_forward_delay=forward_delay)
+    _run_iproute_link('set', device, namespace=namespace, kind='bridge',
+                      br_forward_delay=forward_delay)
 
 
 @privileged.link_cmd.entrypoint
 def set_link_bridge_stp(device, stp, namespace=None):
-    return _run_iproute_link('set', device, namespace=namespace,
-                             kind='bridge', br_stp_state=stp)
+    _run_iproute_link('set', device, namespace=namespace, kind='bridge',
+                      br_stp_state=stp)
 
 
 @privileged.link_cmd.entrypoint
 def set_link_bridge_master(device, bridge, namespace=None):
     bridge_idx = get_link_id(bridge, namespace) if bridge else 0
-    return _run_iproute_link('set', device, namespace=namespace,
-                             master=bridge_idx)
+    _run_iproute_link('set', device, namespace=namespace, master=bridge_idx)
 
 
 @tenacity.retry(
@@ -551,34 +602,11 @@ def remove_netns(name, **kwargs):
 def list_netns(**kwargs):
     """List network namespaces.
 
-    Caller requires raised priveleges to list namespaces
+    Caller requires raised privileges to list namespaces
     """
     return netns.listnetns(**kwargs)
 
 
-def make_serializable(value):
-    """Make a pyroute2 object serializable
-
-    This function converts 'netlink.nla_slot' object (key, value) in a list
-    of two elements.
-    """
-    def _ensure_string(value):
-        return value.decode() if isinstance(value, bytes) else value
-
-    if isinstance(value, list):
-        return [make_serializable(item) for item in value]
-    elif isinstance(value, netlink.nla_slot):
-        return [_ensure_string(value[0]), make_serializable(value[1])]
-    elif isinstance(value, netlink.nla_base):
-        return make_serializable(value.dump())
-    elif isinstance(value, dict):
-        return {_ensure_string(key): make_serializable(data)
-                for key, data in value.items()}
-    elif isinstance(value, tuple):
-        return tuple(make_serializable(item) for item in value)
-    return _ensure_string(value)
-
-
 @tenacity.retry(
     retry=tenacity.retry_if_exception_type(
         netlink_exceptions.NetlinkDumpInterrupted),
@@ -591,9 +619,10 @@ def get_link_devices(namespace, **kwargs):
 
     :return: (list) interfaces in a namespace
     """
+    index = kwargs.pop('index') if 'index' in kwargs else 'all'
     try:
         with get_iproute(namespace) as ip:
-            return make_serializable(ip.get_links(**kwargs))
+            return priv_linux.make_serializable(ip.get_links(index, **kwargs))
     except OSError as e:
         if e.errno == errno.ENOENT:
             raise NetworkNamespaceNotFound(netns_name=namespace)
@@ -629,7 +658,7 @@ def get_ip_addresses(namespace, **kwargs):
     """
     try:
         with get_iproute(namespace) as ip:
-            return make_serializable(ip.get_addr(**kwargs))
+            return priv_linux.make_serializable(ip.get_addr(**kwargs))
     except OSError as e:
         if e.errno == errno.ENOENT:
             raise NetworkNamespaceNotFound(netns_name=namespace)
@@ -647,7 +676,7 @@ def list_ip_rules(namespace, ip_version, match=None, **kwargs):
     """List all IP rules"""
     try:
         with get_iproute(namespace) as ip:
-            rules = make_serializable(ip.get_rules(
+            rules = priv_linux.make_serializable(ip.get_rules(
                 family=_IP_VERSION_FAMILY_MAP[ip_version],
                 match=match, **kwargs))
             for rule in rules:
@@ -718,6 +747,8 @@ def _make_pyroute2_route_args(namespace, ip_version, cidr, device, via, table,
     if metric:
         args['priority'] = int(metric)
     if protocol:
+        if isinstance(protocol, str) and protocol in rtnl.rt_proto:
+            protocol = rtnl.rt_proto[protocol]
         args['proto'] = protocol
     if isinstance(via, (list, tuple)):
         args['multipath'] = []
@@ -741,8 +772,8 @@ def _make_pyroute2_route_args(namespace, ip_version, cidr, device, via, table,
 
 @privileged.default.entrypoint
 def add_ip_route(namespace, cidr, ip_version, device=None, via=None,
-                 table=None, metric=None, scope=None, proto='static',
-                 **kwargs):
+                 table=None, metric=None, scope=None,
+                 proto=rtnl.rt_proto['static'], **kwargs):
     """Add an IP route"""
     kwargs.update(_make_pyroute2_route_args(
         namespace, ip_version, cidr, device, via, table, metric, scope,
@@ -766,10 +797,11 @@ def add_ip_route(namespace, cidr, ip_version, device=None, via=None,
 def list_ip_routes(namespace, ip_version, device=None, table=None, **kwargs):
     """List IP routes"""
     kwargs.update(_make_pyroute2_route_args(
-        namespace, ip_version, None, device, None, table, None, None, None))
+        namespace, ip_version, None, device, None, table, None, 'universe',
+        None))
     try:
         with get_iproute(namespace) as ip:
-            return make_serializable(ip.route('show', **kwargs))
+            return priv_linux.make_serializable(ip.route('show', **kwargs))
     except OSError as e:
         if e.errno == errno.ENOENT:
             raise NetworkNamespaceNotFound(netns_name=namespace)
@@ -803,7 +835,7 @@ def list_bridge_fdb(namespace=None, **kwargs):
     # NOTE(ralonsoh): fbd does not support ifindex filtering in pyroute2 0.5.14
     try:
         with get_iproute(namespace) as ip:
-            return make_serializable(ip.fdb('dump', **kwargs))
+            return priv_linux.make_serializable(ip.fdb('dump', **kwargs))
     except OSError as e:
         if e.errno == errno.ENOENT:
             raise NetworkNamespaceNotFound(netns_name=namespace)
@@ -818,7 +850,7 @@ def _command_bridge_fdb(command, mac, device, dst_ip=None, namespace=None,
         if dst_ip:
             kwargs['dst'] = dst_ip
         with get_iproute(namespace) as ip:
-            return make_serializable(ip.fdb(command, **kwargs))
+            return priv_linux.make_serializable(ip.fdb(command, **kwargs))
     except OSError as e:
         if e.errno == errno.ENOENT:
             raise NetworkNamespaceNotFound(netns_name=namespace)
@@ -828,26 +860,26 @@ def _command_bridge_fdb(command, mac, device, dst_ip=None, namespace=None,
 @privileged.default.entrypoint
 def add_bridge_fdb(mac, device, dst_ip=None, namespace=None, **kwargs):
     """Add a FDB entry"""
-    return _command_bridge_fdb('add', mac, device, dst_ip=dst_ip,
-                               namespace=namespace, **kwargs)
+    _command_bridge_fdb('add', mac, device, dst_ip=dst_ip,
+                        namespace=namespace, **kwargs)
 
 
 @privileged.default.entrypoint
 def append_bridge_fdb(mac, device, dst_ip=None, namespace=None, **kwargs):
     """Add a FDB entry"""
-    return _command_bridge_fdb('append', mac, device, dst_ip=dst_ip,
-                               namespace=namespace, **kwargs)
+    _command_bridge_fdb('append', mac, device, dst_ip=dst_ip,
+                        namespace=namespace, **kwargs)
 
 
 @privileged.default.entrypoint
 def replace_bridge_fdb(mac, device, dst_ip=None, namespace=None, **kwargs):
     """Add a FDB entry"""
-    return _command_bridge_fdb('replace', mac, device, dst_ip=dst_ip,
-                               namespace=namespace, **kwargs)
+    _command_bridge_fdb('replace', mac, device, dst_ip=dst_ip,
+                        namespace=namespace, **kwargs)
 
 
 @privileged.default.entrypoint
 def delete_bridge_fdb(mac, device, dst_ip=None, namespace=None, **kwargs):
     """Add a FDB entry"""
-    return _command_bridge_fdb('del', mac, device, dst_ip=dst_ip,
-                               namespace=namespace, **kwargs)
+    _command_bridge_fdb('del', mac, device, dst_ip=dst_ip,
+                        namespace=namespace, **kwargs)
