@@ -44,6 +44,7 @@ from neutron.conf.agent import common as config
 from neutron.db import db_base_plugin_common as db_base
 from neutron.plugins.ml2.drivers.linuxbridge.agent import \
     linuxbridge_neutron_agent as linuxbridge_agent
+from neutron.services.trunk.drivers.openvswitch.agent import trunk_manager
 from neutron.tests.common import base as common_base
 from neutron.tests.common import helpers
 from neutron.tests import tools
@@ -107,13 +108,20 @@ def set_namespace_gateway(port_dev, gateway_ip):
     port_dev.route.add_gateway(gateway_ip)
 
 
-def assert_ping(src_namespace, dst_ip, timeout=1, count=3):
+def assert_ping(src_namespace, dst_ip, timeout=1, count=3, retry_count=1):
     ipversion = netaddr.IPAddress(dst_ip).version
     ping_command = 'ping' if ipversion == 4 else 'ping6'
     ns_ip_wrapper = ip_lib.IPWrapper(src_namespace)
-    ns_ip_wrapper.netns.execute(
-        [ping_command, '-W', timeout, '-c', count, dst_ip],
-        privsep_exec=True)
+    while retry_count:
+        retry_count -= 1
+        try:
+            ns_ip_wrapper.netns.execute(
+                [ping_command, '-W', timeout, '-c', count, dst_ip],
+                privsep_exec=True)
+            return
+        except n_exc.ProcessExecutionError as exc:
+            if not retry_count:
+                raise exc
 
 
 def assert_async_ping(src_namespace, dst_ip, timeout=1, count=1, interval=1):
@@ -363,7 +371,7 @@ class RootHelperProcess(subprocess.Popen):
                 pass
             msg = ("Process %(cmd)s hasn't been spawned in %(seconds)d "
                    "seconds. Return code: %(ret_code)s, stdout: %(stdout)s, "
-                   "sdterr: %(stderr)s" %
+                   "stderr: %(stderr)s" %
                    {'cmd': self.cmd, 'seconds': timeout,
                     'ret_code': self.returncode, 'stdout': stdout,
                     'stderr': stderr})
@@ -568,7 +576,7 @@ class NetcatTester(object):
             self.server_process.writeline(message)
             message = self.client_process.read_stdout(READ_TIMEOUT).strip()
         except ConnectionError as e:
-            LOG.debug("Error: %s occured during connectivity test.", e)
+            LOG.debug("Error: %s occurred during connectivity test.", e)
             message = ""
 
         return message == testing_string
@@ -825,6 +833,18 @@ class OVSTrunkBridgeFixture(OVSBridgeFixture):
         self.addCleanup(self.bridge.destroy)
 
 
+class OVSTrunkBridgeFixtureTrunkBridge(fixtures.Fixture):
+
+    def __init__(self, trunk_id):
+        super(OVSTrunkBridgeFixtureTrunkBridge, self).__init__()
+        self.trunk_id = trunk_id
+
+    def _setUp(self):
+        self.bridge = trunk_manager.TrunkBridge(self.trunk_id)
+        self.bridge.create()
+        self.addCleanup(self.bridge.destroy)
+
+
 class OVSPortFixture(PortFixture):
     NIC_NAME_LEN = 14
 
