@@ -17,14 +17,13 @@ import collections
 from unittest import mock
 
 from neutron_lib import constants as const
+from neutron_lib.plugins.ml2 import ovs_constants as agent_const
 from oslo_config import cfg
 from ovsdbapp.backend.ovs_idl import idlutils
 
 from neutron.agent.common import ovs_lib
 from neutron.agent.linux import ip_lib
 from neutron.common import utils
-from neutron.plugins.ml2.drivers.openvswitch.agent.common import (
-    constants as agent_const)
 from neutron.tests.common import net_helpers
 from neutron.tests.functional.agent.linux import base
 
@@ -218,11 +217,15 @@ class OVSBridgeTestCase(OVSBridgeTestBase):
         self._test_set_igmp_snooping_state(False)
 
     def test_get_datapath_id(self):
+        def _check_datapath_id():
+            return True if dpid in self.br.get_datapath_id() else False
+
         brdev = ip_lib.IPDevice(self.br.br_name)
         dpid = brdev.link.attributes['link/ether'].replace(':', '')
         self.br.set_db_attribute('Bridge',
                                  self.br.br_name, 'datapath_id', dpid)
-        self.assertIn(dpid, self.br.get_datapath_id())
+        # Wait until local OVS DB cache is updated.
+        utils.wait_until_true(_check_datapath_id, timeout=5)
 
     def _test_add_tunnel_port(self, attrs,
                               expected_tunnel_type=const.TYPE_GRE):
