@@ -27,6 +27,7 @@ import sqlalchemy
 from sqlalchemy import event  # noqa
 from sqlalchemy.sql import ddl as sqla_ddl
 
+from neutron.common import config
 from neutron.db import migration as migration_root
 from neutron.db.migration.alembic_migrations import external
 from neutron.db.migration import cli as migration
@@ -35,7 +36,6 @@ from neutron.tests import base as test_base
 from neutron.tests.functional import base as functional_base
 from neutron.tests.unit import testlib_api
 
-cfg.CONF.import_opt('core_plugin', 'neutron.conf.common')
 
 CREATION_OPERATIONS = {
     'sqla': (sqla_ddl.CreateIndex,
@@ -143,6 +143,7 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
     TIMEOUT_SCALING_FACTOR = 4
 
     def setUp(self):
+        config.register_common_config_options()
         super(_TestModelsMigrations, self).setUp()
         self.cfg = self.useFixture(config_fixture.Config())
         self.cfg.config(core_plugin='ml2')
@@ -249,7 +250,7 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
                 if not get_excepted_elements:
                     continue
                 explanation = getattr(get_excepted_elements, '__doc__', "")
-                if len(explanation) < 1:
+                if not explanation:
                     self.fail("%s() requires docstring with explanation" %
                               '.'.join([m.module.__name__,
                                         get_excepted_elements.__name__]))
@@ -392,7 +393,7 @@ class TestSanityCheck(testlib_api.SqlTestCaseLight):
             sqlalchemy.Column('router_id', sqlalchemy.String(36)),
             sqlalchemy.Column('l3_agent_id', sqlalchemy.String(36)))
 
-        with self.engine.connect() as conn:
+        with self.engine.begin() as conn:
             ha_router_agent_port_bindings.create(conn)
             self.addCleanup(self._drop_table, ha_router_agent_port_bindings)
             # NOTE(haleyb): without this disabled, pylint complains
@@ -417,7 +418,7 @@ class TestSanityCheck(testlib_api.SqlTestCaseLight):
             sqlalchemy.Column('port_id', sqlalchemy.String(36)),
             sqlalchemy.Column('port_type', sqlalchemy.String(255)))
 
-        with self.engine.connect() as conn:
+        with self.engine.begin() as conn:
             routerports.create(conn)
             self.addCleanup(self._drop_table, routerports)
             # NOTE(haleyb): without this disabled, pylint complains
@@ -442,7 +443,7 @@ class TestSanityCheck(testlib_api.SqlTestCaseLight):
             sqlalchemy.Column('fixed_port_id', sqlalchemy.String(36)),
             sqlalchemy.Column('fixed_ip_address', sqlalchemy.String(64)))
 
-        with self.engine.connect() as conn:
+        with self.engine.begin() as conn:
             floatingips.create(conn)
             self.addCleanup(self._drop_table, floatingips)
             # NOTE(haleyb): without this disabled, pylint complains
@@ -469,7 +470,7 @@ class TestSanityCheck(testlib_api.SqlTestCaseLight):
             sqlalchemy.Column('fixed_port_id', sqlalchemy.String(36)),
             sqlalchemy.Column('fixed_ip_address', sqlalchemy.String(64)))
 
-        with self.engine.connect() as conn:
+        with self.engine.begin() as conn:
             floatingips.create(conn)
             self.addCleanup(self._drop_table, floatingips)
             # NOTE(haleyb): without this disabled, pylint complains
