@@ -30,6 +30,12 @@ class LogApiTestCaseBase(functional_base.TestOVNFunctionalBase):
         self._check_is_supported()
         self.ctxt = context.Context('admin', 'fake_tenant')
 
+        # Since these tests use the _create_network() from the unit test suite
+        # but _create_security_group() is from the functional tests, two
+        # different tenant_ids will be used unless we specify the following
+        # line in the code:
+        self._tenant_id = self.ctxt.project_id
+
     def _check_is_supported(self):
         if not self.log_driver.network_logging_supported(self.nb_api):
             self.skipTest("The current OVN version does not offer support "
