@@ -23,6 +23,7 @@ import neutron_lib
 from neutron_lib.api.definitions import extra_dhcp_opt as edo_ext
 from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants as n_const
+from neutron_lib import exceptions as n_exc
 from oslo_concurrency import processutils
 from oslo_config import cfg
 import testtools
@@ -588,8 +589,8 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             'parent_name': 'fake-parent-port-uuid',
             'tag': 42
         }
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, portbindings.VNIC_NORMAL, []),
             utils.validate_and_get_data_from_binding_profile(
                 {constants.OVN_PORT_BINDING_PROFILE: expect}))
 
@@ -597,8 +598,8 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             'vtep-physical-switch': 'fake-physical-switch-uuid',
             'vtep-logical-switch': 'fake-logical-switch-uuid',
         }
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, portbindings.VNIC_NORMAL, []),
             utils.validate_and_get_data_from_binding_profile(
                 {constants.OVN_PORT_BINDING_PROFILE: expect}))
 
@@ -611,8 +612,9 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
         }
         expect = binding_profile.copy()
         del(expect[constants.PORT_CAP_PARAM])
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, portbindings.VNIC_DIRECT,
+                         [constants.PORT_CAP_SWITCHDEV]),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: portbindings.VNIC_DIRECT,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
@@ -626,8 +628,9 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
         }
         expect = binding_profile.copy()
         del(expect[constants.PORT_CAP_PARAM])
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, portbindings.VNIC_DIRECT,
+                         [constants.PORT_CAP_SWITCHDEV]),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: portbindings.VNIC_DIRECT,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
@@ -640,11 +643,11 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             'pf_mac_address': '00:53:00:00:00:42',
             'vf_num': 42,
         }
-        self.assertDictEqual(
+        self.assertEqual(
+            utils.BPInfo(expect, portbindings.VNIC_REMOTE_MANAGED, []),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: portbindings.VNIC_REMOTE_MANAGED,
-                 constants.OVN_PORT_BINDING_PROFILE: expect}),
-            expect)
+                 constants.OVN_PORT_BINDING_PROFILE: expect}))
 
     def test_valid_input_surplus_keys(self):
         # Confirm that extra keys are allowed
@@ -658,8 +661,9 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
         expect = binding_profile.copy()
         del(expect[constants.PORT_CAP_PARAM])
         del(expect['optional_information_provided_by_nova'])
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, portbindings.VNIC_DIRECT,
+                         [constants.PORT_CAP_SWITCHDEV]),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: portbindings.VNIC_DIRECT,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
@@ -667,7 +671,7 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
     def test_unknown_profile_items_pruned(self):
         # Confirm that unknown profile items are pruned
         self.assertEqual(
-            {},
+            utils.BPInfo({}, portbindings.VNIC_NORMAL, []),
             utils.validate_and_get_data_from_binding_profile(
                 {constants.OVN_PORT_BINDING_PROFILE: {
                     'unknown-key': 'unknown-data'}}))
@@ -676,16 +680,16 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
         expect = {
             'key': 'value',
         }
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, self.VNIC_FAKE_NORMAL, []),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_NORMAL,
                  constants.OVN_PORT_BINDING_PROFILE: expect}))
         expect = {
             'key': None,
         }
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, self.VNIC_FAKE_NORMAL, []),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_NORMAL,
                  constants.OVN_PORT_BINDING_PROFILE: expect}))
@@ -706,17 +710,17 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             'other_key': 'value',
         }
         # This param set is valid for VNIC_FAKE_NORMAL with 'other_key' pruned.
-        expect = binding_profile.copy()
-        del(expect['other_key'])
-        self.assertDictEqual(
-            expect,
+        expected_bp = binding_profile.copy()
+        del(expected_bp['other_key'])
+        self.assertEqual(
+            utils.BPInfo(expected_bp, self.VNIC_FAKE_NORMAL, []),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_NORMAL,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
         # It is valid for VNIC_FAKE_OTHER
-        expect = binding_profile.copy()
-        self.assertDictEqual(
-            expect,
+        expected_bp = binding_profile.copy()
+        self.assertEqual(
+            utils.BPInfo(expected_bp, self.VNIC_FAKE_OTHER, []),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_OTHER,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
@@ -744,7 +748,7 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             'third_key': 'value',
         }
         self.assertEqual(
-            {},
+            utils.BPInfo({}, self.VNIC_FAKE_OTHER, ['fake-capability']),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_OTHER,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
@@ -757,8 +761,9 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
         }
         expect = binding_profile.copy()
         del(expect[constants.PORT_CAP_PARAM])
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, self.VNIC_FAKE_OTHER,
+                         [constants.PORT_CAP_SWITCHDEV]),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_OTHER,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
@@ -775,7 +780,8 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             constants.PORT_CAP_PARAM: [constants.PORT_CAP_SWITCHDEV],
         }
         self.assertEqual(
-            {},
+            utils.BPInfo({}, self.VNIC_FAKE_OTHER,
+                         [constants.PORT_CAP_SWITCHDEV]),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_OTHER,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
@@ -938,3 +944,203 @@ class TestOvsdbClientCommand(base.BaseTestCase):
     def test_run_bad_schema(self):
         with testtools.ExpectedException(KeyError):
             self.OvsdbClientTestCommand.run(['foo'])
+
+
+class GetSubnetsAddressScopeTestCase(base.BaseTestCase):
+
+    def setUp(self):
+        super().setUp()
+        self.ml2_plugin = mock.Mock()
+
+    def test_no_subnets(self):
+        subnets = []
+        fixed_ips = mock.ANY
+        address4, address6 = utils.get_subnets_address_scopes(
+            mock.ANY, subnets, fixed_ips, self.ml2_plugin)
+        self.assertEqual(('', ''), (address4, address6))
+
+    def test_no_subnetpool(self):
+        subnets = [
+            {'id': 'subnet1', 'subnetpool_id': None},
+            {'id': 'subnet2', 'subnetpool_id': None},
+        ]
+        fixed_ips = [
+            {'subnet_id': 'subnet1'},
+            {'subnet_id': 'subnet2'},
+        ]
+        address4, address6 = utils.get_subnets_address_scopes(
+            mock.ANY, subnets, fixed_ips, self.ml2_plugin)
+        self.assertEqual(('', ''), (address4, address6))
+
+    def test_no_address_scope(self):
+        subnets = [
+            {'id': 'subnet1', 'subnetpool_id': 'pool_ipv4'},
+            {'id': 'subnet2', 'subnetpool_id': 'pool_ipv6'},
+        ]
+        fixed_ips = [
+            {'subnet_id': 'subnet1'},
+            {'subnet_id': 'subnet2'},
+        ]
+        self.ml2_plugin.get_subnetpool.side_effect = n_exc.SubnetPoolNotFound(
+            subnetpool_id='snp')
+        address4, address6 = utils.get_subnets_address_scopes(
+            mock.ANY, subnets, fixed_ips, self.ml2_plugin)
+        self.assertEqual(('', ''), (address4, address6))
+
+    def test_address_scope(self):
+        subnets = [
+            {'id': 'subnet1', 'subnetpool_id': 'pool_ipv4'},
+            {'id': 'subnet2', 'subnetpool_id': 'pool_ipv6'},
+        ]
+        fixed_ips = [
+            {'subnet_id': 'subnet1'},
+            {'subnet_id': 'subnet2'},
+        ]
+        self.ml2_plugin.get_subnetpool.side_effect = [
+            {'address_scope_id': 'scope4', 'ip_version': n_const.IP_VERSION_4},
+            {'address_scope_id': 'scope6', 'ip_version': n_const.IP_VERSION_6},
+        ]
+        address4, address6 = utils.get_subnets_address_scopes(
+            mock.ANY, subnets, fixed_ips, self.ml2_plugin)
+        self.assertEqual(('scope4', 'scope6'), (address4, address6))
+
+
+class GetPortTypeVirtualAndParentsTestCase(base.BaseTestCase):
+
+    def test_no_subnets(self):
+        subnets = []
+        fixed_ips = []
+        port_type, virtual_ip, virtual_parents = (
+            utils.get_port_type_virtual_and_parents(subnets, fixed_ips, 'net1',
+                                                    'port1', mock.ANY))
+        self.assertEqual(('', None, None),
+                         (port_type, virtual_ip, virtual_parents))
+
+    @mock.patch.object(utils, 'get_virtual_port_parents', return_value=[])
+    def test_no_parents(self, *args):
+        subnets = [
+            {'id': 'subnet1'},
+            {'id': 'subnet2'},
+        ]
+        fixed_ips = [
+            {'subnet_id': 'subnet1', 'ip_address': '1.2.3.4'},
+            {'subnet_id': 'subnet2', 'ip_address': '1.2.3.5'},
+        ]
+        port_type, virtual_ip, virtual_parents = (
+            utils.get_port_type_virtual_and_parents(subnets, fixed_ips, 'net1',
+                                                    'port1', mock.ANY))
+        self.assertEqual(('', None, None),
+                         (port_type, virtual_ip, virtual_parents))
+
+    @mock.patch.object(utils, 'get_virtual_port_parents',
+                       return_value=['parent1', 'parent2'])
+    def test_with_parents(self, *args):
+        subnets = [
+            {'id': 'subnet1'},
+            {'id': 'subnet2'},
+        ]
+        fixed_ips = [
+            {'subnet_id': 'subnet1', 'ip_address': '1.2.3.4'},
+            {'subnet_id': 'subnet2', 'ip_address': '1.2.3.5'},
+        ]
+        port_type, virtual_ip, virtual_parents = (
+            utils.get_port_type_virtual_and_parents(subnets, fixed_ips, 'net1',
+                                                    'port1', mock.ANY))
+        self.assertEqual((constants.LSP_TYPE_VIRTUAL, '1.2.3.4',
+                          'parent1,parent2'),
+                         (port_type, virtual_ip, virtual_parents))
+
+
+class DetermineBindHostTestCase(base.BaseTestCase):
+
+    def setUp(self):
+        super().setUp()
+        self.mock_sb_idl = mock.Mock()
+        self.get_chassis_by_card_serial_from_cms_options = (
+            self.mock_sb_idl.get_chassis_by_card_serial_from_cms_options)
+        self.fake_smartnic_hostname = 'fake-chassis-hostname'
+        self.get_chassis_by_card_serial_from_cms_options.return_value = (
+            fakes.FakeChassis.create(
+                attrs={'hostname': self.fake_smartnic_hostname}))
+
+    def test_vnic_normal_unbound_port(self):
+        self.assertEqual(
+            '',
+            utils.determine_bind_host(self.mock_sb_idl, {}))
+
+    def test_vnic_normal_bound_port(self):
+        port = {
+            portbindings.HOST_ID: 'fake-binding-host-id',
+        }
+        self.assertEqual(
+            'fake-binding-host-id',
+            utils.determine_bind_host(self.mock_sb_idl, port))
+
+    def test_vnic_normal_port_context(self):
+        context = mock.MagicMock()
+        context.host = 'fake-binding-host-id'
+        self.assertEqual(
+            'fake-binding-host-id',
+            utils.determine_bind_host(self.mock_sb_idl, {},
+                                      port_context=context))
+
+    def test_vnic_remote_managed_unbound_port_no_binding_profile(self):
+        port = {
+            portbindings.VNIC_TYPE: portbindings.VNIC_REMOTE_MANAGED,
+            constants.OVN_PORT_BINDING_PROFILE: {},
+        }
+        self.assertEqual(
+            '',
+            utils.determine_bind_host(self.mock_sb_idl, port))
+
+    def test_vnic_remote_managed_unbound_port(self):
+        port = {
+            portbindings.VNIC_TYPE: portbindings.VNIC_REMOTE_MANAGED,
+            constants.OVN_PORT_BINDING_PROFILE: {
+                constants.VIF_DETAILS_PCI_VENDOR_INFO: 'fake-pci-vendor-info',
+                constants.VIF_DETAILS_PCI_SLOT: 'fake-pci-slot',
+                constants.VIF_DETAILS_PHYSICAL_NETWORK: None,
+                constants.VIF_DETAILS_CARD_SERIAL_NUMBER: 'fake-serial',
+                constants.VIF_DETAILS_PF_MAC_ADDRESS: 'fake-pf-mac',
+                constants.VIF_DETAILS_VF_NUM: 42,
+            },
+        }
+        self.assertEqual(
+            self.fake_smartnic_hostname,
+            utils.determine_bind_host(self.mock_sb_idl, port))
+
+    def test_vnic_remote_managed_bound_port(self):
+        port = {
+            portbindings.VNIC_TYPE: portbindings.VNIC_REMOTE_MANAGED,
+            portbindings.HOST_ID: 'fake-binding-host-id',
+            constants.OVN_PORT_BINDING_PROFILE: {
+                constants.VIF_DETAILS_PCI_VENDOR_INFO: 'fake-pci-vendor-info',
+                constants.VIF_DETAILS_PCI_SLOT: 'fake-pci-slot',
+                constants.VIF_DETAILS_PHYSICAL_NETWORK: None,
+                constants.VIF_DETAILS_CARD_SERIAL_NUMBER: 'fake-serial',
+                constants.VIF_DETAILS_PF_MAC_ADDRESS: 'fake-pf-mac',
+                constants.VIF_DETAILS_VF_NUM: 42,
+            },
+        }
+        self.assertEqual(
+            self.fake_smartnic_hostname,
+            utils.determine_bind_host(self.mock_sb_idl, port))
+
+    def test_vnic_remote_managed_port_context(self):
+        context = mock.MagicMock()
+        context.current = {
+            portbindings.VNIC_TYPE: portbindings.VNIC_REMOTE_MANAGED,
+            constants.OVN_PORT_BINDING_PROFILE: {
+                constants.VIF_DETAILS_PCI_VENDOR_INFO: 'fake-pci-vendor-info',
+                constants.VIF_DETAILS_PCI_SLOT: 'fake-pci-slot',
+                constants.VIF_DETAILS_PHYSICAL_NETWORK: None,
+                constants.VIF_DETAILS_CARD_SERIAL_NUMBER: 'fake-serial',
+                constants.VIF_DETAILS_PF_MAC_ADDRESS: 'fake-pf-mac',
+                constants.VIF_DETAILS_VF_NUM: 42,
+            },
+        }
+        context.host = 'fake-binding-host-id'
+        self.assertEqual(
+            self.fake_smartnic_hostname,
+            utils.determine_bind_host(self.mock_sb_idl, {},
+                                      port_context=context))
