@@ -88,12 +88,31 @@ class SystemReaderTests(SystemMemberTests):
         self.context = self.system_reader_ctx
 
 
-class ProjectAdminTests(AutoAllocatedTopologyAPITestCase):
+class AdminTests(AutoAllocatedTopologyAPITestCase):
 
     def setUp(self):
-        super(ProjectAdminTests, self).setUp()
+        super(AdminTests, self).setUp()
         self.context = self.project_admin_ctx
 
+    def test_get_topology(self):
+        self.assertTrue(
+            policy.enforce(self.context, GET_POLICY, self.target))
+        self.assertTrue(
+            policy.enforce(self.context, GET_POLICY, self.alt_target))
+
+    def test_delete_topology(self):
+        self.assertTrue(
+            policy.enforce(self.context, DELETE_POLICY, self.target))
+        self.assertTrue(
+            policy.enforce(self.context, DELETE_POLICY, self.alt_target))
+
+
+class ProjectMemberTests(AdminTests):
+
+    def setUp(self):
+        super(ProjectMemberTests, self).setUp()
+        self.context = self.project_member_ctx
+
     def test_get_topology(self):
         self.assertTrue(policy.enforce(self.context, GET_POLICY, self.target))
         self.assertRaises(
@@ -103,7 +122,7 @@ class ProjectAdminTests(AutoAllocatedTopologyAPITestCase):
         )
 
     def test_delete_topology(self):
-        # Project members can delete topolies inside project, but not outside
+        # Project members can delete topologies inside project, but not outside
         # project
         self.assertTrue(
             policy.enforce(self.context, DELETE_POLICY, self.target)
@@ -115,13 +134,6 @@ class ProjectAdminTests(AutoAllocatedTopologyAPITestCase):
         )
 
 
-class ProjectMemberTests(ProjectAdminTests):
-
-    def setUp(self):
-        super(ProjectMemberTests, self).setUp()
-        self.context = self.project_member_ctx
-
-
 class ProjectReaderTests(ProjectMemberTests):
 
     def setUp(self):
